@@ -11,14 +11,18 @@ import getdns, pprint, sys, time
 srvname = sys.argv[1]
 
 ctx = getdns.Context()
-results = ctx.service(name=srvname)
+try:
+    results = ctx.service(name=srvname)
+except getdns.error, e:
+    print(str(e))
+    sys.exit(1)
 
-if results["status"] == getdns.GETDNS_RESPSTATUS_GOOD:
-    for reply in results["replies_tree"]:
+if results.status == getdns.RESPSTATUS_GOOD:
+    for reply in results.replies_tree:
         for a in reply["answer"]:
             rrname  = a["name"]
             rrtype  = a["type"]
-            if rrtype == getdns.GETDNS_RRTYPE_SRV:
+            if rrtype == getdns.RRTYPE_SRV:
                 rdata   = a["rdata"]
                 prio, weight, port, target = rdata['priority'], rdata['weight'], rdata['port'], rdata['target']
                 print "SRV %s --> %d %d %d %s" % \
