@@ -36,9 +36,93 @@
 #include <stdio.h>
 #include <string.h>
 #include <getdns/getdns.h>
+#include <getdns/getdns_extra.h>
 #include <ldns/ldns.h>
 #include "pygetdns.h"
 
+
+
+int
+get_status(struct getdns_dict *result_dict)
+{
+    uint32_t status;
+    getdns_return_t ret;
+
+    if ((ret = getdns_dict_get_int(result_dict, "status", &status)) != GETDNS_RETURN_GOOD)
+        return 0;
+    return (int)status;
+}
+        
+
+int
+get_answer_type(struct getdns_dict *result_dict)
+{
+    uint32_t answer_type;
+    getdns_return_t ret;
+
+    if ((ret = getdns_dict_get_int(result_dict, "answer_type", &answer_type)) != GETDNS_RETURN_GOOD)
+        return 0;
+    return (int)answer_type;
+}
+
+
+char *
+get_canonical_name(struct getdns_dict *result_dict)
+{
+    getdns_bindata *canonical_name;
+    getdns_return_t ret;
+
+    if ((ret = getdns_dict_get_bindata(result_dict, "canonical_name", &canonical_name)) == GETDNS_RETURN_GOOD)  {
+        char *dname = 0;
+        if (getdns_convert_dns_name_to_fqdn(canonical_name, &dname) == GETDNS_RETURN_GOOD)
+            return dname;
+        else
+            return (char *)canonical_name->data;
+    }  else
+        return 0;
+}
+        
+
+PyObject *
+get_just_address_answers(struct getdns_dict *result_dict)
+{
+    struct getdns_list *just_address_answers;
+    getdns_return_t ret;
+
+    if ((ret = getdns_dict_get_list(result_dict, "just_address_answers", &just_address_answers)) !=
+        GETDNS_RETURN_GOOD)
+        return NULL;
+    return pythonify_address_list(just_address_answers);
+}
+
+
+PyObject *
+get_replies_tree(struct getdns_dict *result_dict)
+{
+    struct getdns_list *replies_tree;
+    getdns_return_t ret;
+
+    if ((ret = getdns_dict_get_list(result_dict, "replies_tree", &replies_tree)) !=
+        GETDNS_RETURN_GOOD)
+        return NULL;
+    return glist_to_plist(replies_tree);
+}
+
+
+PyObject *
+get_validation_chain(struct getdns_dict *result_dict)
+{
+    struct getdns_list *validation_chain;
+    getdns_return_t ret;
+
+    if ((ret = getdns_dict_get_list(result_dict, "validation_chain", &validation_chain)) !=
+        GETDNS_RETURN_GOOD)
+        Py_RETURN_NONE;
+    else
+        return glist_to_plist(validation_chain);
+}
+
+
 struct getdns_dict *
 extensions_to_getdnsdict(PyDictObject *pydict)
 {
@@ -54,7 +138,7 @@ extensions_to_getdnsdict(PyDictObject *pydict)
     struct getdns_bindata *option_data;
     struct getdns_dict *tmpoptions_list_dict; /* a dict to hold add_opt_parameters[options] stuff */
 
-    if (!PyDict_Check(pydict))  {
+    if ((!pydict) || (!PyDict_Check(pydict)))  {
         PyErr_SetString(getdns_error, "Expected dict, didn't get one");
         return NULL;
     }
@@ -270,17 +354,13 @@ pythonify_address_list(getdns_list *list)
     char paddr_buf[256];
 
     if ((ret = getdns_list_get_length(list, &length)) != GETDNS_RETURN_GOOD)  {
-        char err_buf[256];
-        getdns_strerror(ret, err_buf, sizeof err_buf);
-        PyErr_SetString(getdns_error, err_buf);
+        PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
         return NULL;
     }
     py_list = PyList_New(0);
     for ( i = 0 ; i < (int)length ; i++ )  {
         if ((ret = getdns_list_get_data_type(list, i, &type)) != GETDNS_RETURN_GOOD)  {
-            char err_buf[256];
-            getdns_strerror(ret, err_buf, sizeof err_buf);
-            PyErr_SetString(getdns_error, err_buf);
+            PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
             return NULL;
         }
         if (type != t_dict)  {
@@ -288,23 +368,17 @@ pythonify_address_list(getdns_list *list)
             return NULL;
         }
         if ((ret = getdns_list_get_dict(list, (size_t)i, &a_item)) != GETDNS_RETURN_GOOD)  {
-            char err_buf[256];
-            getdns_strerror(ret, err_buf, sizeof err_buf);
-            PyErr_SetString(getdns_error, err_buf);
+            PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
             return NULL;
         }
         if ((ret = getdns_dict_get_bindata(a_item, "address_type", &a_address_type)) !=
             GETDNS_RETURN_GOOD)  {
-            char err_buf[256];
-            getdns_strerror(ret, err_buf, sizeof err_buf);
-            PyErr_SetString(getdns_error, err_buf);
+            PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
             return NULL;
         }
         if ((ret = getdns_dict_get_bindata(a_item, "address_data", &a_address_data)) !=
             GETDNS_RETURN_GOOD)  {
-            char err_buf[256];
-            getdns_strerror(ret, err_buf, sizeof err_buf);
-            PyErr_SetString(getdns_error, err_buf);
+            PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
             return NULL;
         }
         if (!strncasecmp((char *)a_address_type->data, "IPv4", 4))
@@ -349,9 +423,7 @@ glist_to_plist(struct getdns_list *list)
     }
         
     if ((ret = getdns_list_get_length(list, &count)) != GETDNS_RETURN_GOOD)  {
-        char err_buf[256];
-        getdns_strerror(ret, err_buf, sizeof err_buf);
-        PyErr_SetString(getdns_error, err_buf);
+        PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
         return NULL;
     }
     py_list = PyList_New(0);
@@ -360,9 +432,7 @@ glist_to_plist(struct getdns_list *list)
         switch (type)  {
         case t_dict:
             if ((ret = getdns_list_get_dict(list, i, &dict_item)) != GETDNS_RETURN_GOOD)  {
-                char err_buf[256];
-                getdns_strerror(ret, err_buf, sizeof err_buf);
-                PyErr_SetString(getdns_error, err_buf);
+                PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
                 return NULL;
             }
             if ((py_dict = gdict_to_pdict(dict_item)) == NULL)  {
@@ -377,15 +447,11 @@ glist_to_plist(struct getdns_list *list)
 
         case t_list:
             if ((ret = getdns_list_get_list(list, i, &list_item)) != GETDNS_RETURN_GOOD)  {
-                char err_buf[256];
-                getdns_strerror(ret, err_buf, sizeof err_buf);
-                PyErr_SetString(getdns_error, err_buf);
+                PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
                 return NULL;
             }
             if ((py_locallist = glist_to_plist(list_item)) == NULL)  {
-                char err_buf[256];
-                getdns_strerror(ret, err_buf, sizeof err_buf);
-                PyErr_SetString(getdns_error, err_buf);
+                PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
                 return NULL;
             }
             if (PyList_Append(py_list, py_locallist) == -1)  {
@@ -396,9 +462,7 @@ glist_to_plist(struct getdns_list *list)
 
         case t_int:
             if ((ret = getdns_list_get_int(list, i, &localint)) != GETDNS_RETURN_GOOD)  {
-                char err_buf[256];
-                getdns_strerror(ret, err_buf, sizeof err_buf);
-                PyErr_SetString(getdns_error, err_buf);
+                PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
                 return NULL;
             }
             py_int = PyInt_FromLong((long)localint);
@@ -410,9 +474,7 @@ glist_to_plist(struct getdns_list *list)
 
         case t_bindata:
             if ((ret = getdns_list_get_bindata(list, i, &data)) != GETDNS_RETURN_GOOD)  {
-                char err_buf[256];
-                getdns_strerror(ret, err_buf, sizeof err_buf);
-                PyErr_SetString(getdns_error, err_buf);
+                PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
                 return NULL;
             }
             if ((py_bindata = convertBinData(data, "")) == 0)  {
@@ -452,18 +514,14 @@ gdict_to_pdict(struct getdns_dict *dict)
     PyObject *py_localbindata;
 
     if ((ret = getdns_dict_get_names(dict, &keys)) != GETDNS_RETURN_GOOD)  {
-        char err_buf[256];
-        getdns_strerror(ret, err_buf, sizeof err_buf);
-        PyErr_SetString(getdns_error, err_buf);
+        PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
         return NULL;
     }
     py_dict = PyDict_New();
     (void)getdns_list_get_length(keys, &n_keys);
     for (i = 0 ; i < (int)n_keys ; i++ )  {
         if ((ret = getdns_list_get_bindata(keys, (size_t)i, &key_name)) != GETDNS_RETURN_GOOD)  {
-            char err_buf[256];
-            getdns_strerror(ret, err_buf, sizeof err_buf);
-            PyErr_SetString(getdns_error, err_buf);
+            PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
             return NULL;
         }
         if (strnlen((char *)key_name->data, 256) == 256)  { /* too long, something's wrong */
@@ -472,18 +530,14 @@ gdict_to_pdict(struct getdns_dict *dict)
         }
         if ((ret = getdns_dict_get_data_type(dict, (char *)key_name->data, &type)) !=
             GETDNS_RETURN_GOOD)  {
-            char err_buf[256];
-            getdns_strerror(ret, err_buf, sizeof err_buf);
-            PyErr_SetString(getdns_error, err_buf);
+            PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
             return NULL;
         }
         switch (type)  {
         case t_dict:
             if ((ret = getdns_dict_get_dict(dict, (char *)key_name->data, &dict_item)) !=
                 GETDNS_RETURN_GOOD)  {
-                char err_buf[256];
-                getdns_strerror(ret, err_buf, sizeof err_buf);
-                PyErr_SetString(getdns_error, err_buf);
+                PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
                 return NULL;
             }
             if ((py_localdict = gdict_to_pdict(dict_item)) == NULL)  {
@@ -499,9 +553,7 @@ gdict_to_pdict(struct getdns_dict *dict)
         case t_list:
             if ((ret = getdns_dict_get_list(dict, (char *)key_name->data, &list_item)) !=
                 GETDNS_RETURN_GOOD)  {
-                char err_buf[256];
-                getdns_strerror(ret, err_buf, sizeof err_buf);
-                PyErr_SetString(getdns_error, err_buf);
+                PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
                 return NULL;
             }
             if ((py_locallist = glist_to_plist(list_item)) == NULL)  {
@@ -517,9 +569,7 @@ gdict_to_pdict(struct getdns_dict *dict)
         case t_int:
             if ((ret = getdns_dict_get_int(dict, (char *)key_name->data, &int_item)) !=
                 GETDNS_RETURN_GOOD)  {
-                char err_buf[256];
-                getdns_strerror(ret, err_buf, sizeof err_buf);
-                PyErr_SetString(getdns_error, err_buf);
+                PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
                 return NULL;
             }
             py_localint = PyInt_FromLong((long)int_item);
@@ -532,9 +582,7 @@ gdict_to_pdict(struct getdns_dict *dict)
         case t_bindata:
             if ((ret = getdns_dict_get_bindata(dict, (char *)key_name->data, &bindata_item))
                 != GETDNS_RETURN_GOOD)  {
-                char err_buf[256];
-                getdns_strerror(ret, err_buf, sizeof err_buf);
-                PyErr_SetString(getdns_error, err_buf);
+                PyErr_SetString(getdns_error, getdns_get_errorstr_by_id(ret));
                 return NULL;
             }
             if ((py_localbindata = convertBinData(bindata_item, "")) == 0)  {
@@ -553,49 +601,7 @@ gdict_to_pdict(struct getdns_dict *dict)
     return py_dict;
 }
 
-#if 0
 
-PyObject *
-decode_getdns_response(struct getdns_dict *response)
-{
-    uint32_t error;
-    char error_str[512];
-    struct getdns_list *addr_list;
-    size_t n_addrs;
-    getdns_return_t ret;
-    size_t i;
-    struct getdns_dict *addr_dict;
-    struct getdns_bindata *addr;
-    char *addr_str;
-    PyObject *results;
-
-    (void)getdns_dict_get_int(response, "status", &error);
-    if (error != GETDNS_RESPSTATUS_GOOD)  {
-        sprintf(error_str, "No answers, return value %d", error);
-        PyErr_SetString(getdns_error, error_str);
-        return NULL;
-    }
-    if ((ret = getdns_dict_get_list(response, "just_address_answers",
-                                    &addr_list)) != GETDNS_RETURN_GOOD)  {
-        sprintf(error_str, "Extracting answers failed: %d", ret);
-        PyErr_SetString(getdns_error, error_str);
-        return NULL;
-    }
-    if ((results = PyList_New(0)) == NULL)  {
-        PyErr_SetString(getdns_error, "Unable to allocate response list");
-        return NULL;
-    }
-    (void)getdns_list_get_length(addr_list, &n_addrs);
-    for ( i = 0 ; i < n_addrs ; i++ )  {
-        (void)getdns_list_get_dict(addr_list, i, &addr_dict);
-        (void)getdns_dict_get_bindata(addr_dict, "address_data", &addr);
-        addr_str = getdns_display_ip_address(addr);
-        PyList_Append(results, PyString_FromString(addr_str));
-    }
-    return results;
-}
-
-#endif
 /*
  * Error checking helper
  */
@@ -611,39 +617,6 @@ void error_exit(char* msg, getdns_return_t ret)
 
 
 
-/**
- * reverse an IP address for PTR lookup
- * @param address_data IP address to reverse
- * @return NULL on allocation failure
- * @return reversed string on success, caller must free storage via call to free()
- */
-char *
-reverse_address(struct getdns_bindata *address_data)
-{
-    ldns_rdf *addr_rdf;
-    ldns_rdf *rev_rdf;
-    char *rev_str;
-
-    if (address_data->size == 4)
-        addr_rdf = ldns_rdf_new(LDNS_RDF_TYPE_A, 4, address_data->data);
-    else if (address_data->size == 16)
-        addr_rdf = ldns_rdf_new(LDNS_RDF_TYPE_AAAA, 16, address_data->data);
-    else
-        return NULL;
-    if (!addr_rdf)
-        return NULL;
-
-    rev_rdf = ldns_rdf_address_reverse(addr_rdf);
-    ldns_rdf_free(addr_rdf);
-    if (!rev_rdf)
-        return NULL;
-
-    rev_str = ldns_rdf2str(rev_rdf);
-    ldns_rdf_deep_free(rev_rdf);
-    return rev_str;
-}
-
-
 // Code to display the entire response.
 // answer_type
 // canonical_name
@@ -670,9 +643,8 @@ priv_getdns_bindata_is_dname(struct getdns_bindata *bindata)
 
 PyObject *
 convertBinData(getdns_bindata* data,
-                    const char* key) {
-
-
+                    const char* key) 
+{
     size_t i; 
 
 
@@ -727,9 +699,12 @@ convertBinData(getdns_bindata* data,
             PyErr_SetString(getdns_error, GETDNS_RETURN_GENERIC_ERROR_TEXT);
             return NULL;
         }
-    } else if (key != NULL &&
+    } else if (key != NULL && (strcmp(key, "address_data") == 0))  { /* XXX */
+               
+#if 0
         (strcmp(key, "ipv4_address") == 0 ||
          strcmp(key, "ipv6_address") == 0)) {
+#endif
         char* ipStr = getdns_display_ip_address(data);
         if (ipStr) {
             PyObject *addr_string;
@@ -740,28 +715,16 @@ convertBinData(getdns_bindata* data,
             return(addr_string);
         }
     }  else  {                  /* none of the above, treat it like a blob */
-#if 0
-        Py_buffer pybuf;
-#endif
         uint8_t *blob = (uint8_t *)malloc(data->size);
-
+        
         memcpy(blob, data->data, data->size);
         return (PyBuffer_FromMemory(blob, (Py_ssize_t)data->size));
-#if 0
-        if (PyBuffer_FillInfo(&pybuf, 0, blob, data->size, false, PyBUF_CONTIG) < 0)  {
-            PyErr_SetString(getdns_error, "PyBuffer_FillInfo failed\n");
-            return NULL;
-        }
-        return PyMemoryView_FromBuffer(&pybuf);
-#endif
     }
     return NULL;                /* should never get here .. */
 }
 
-char* 
-getdns_dict_to_ip_string(getdns_dict* dict);
-PyObject* 
-convertToList(struct getdns_list* list);
+PyObject *getdns_dict_to_ip_string(getdns_dict* dict);
+PyObject *convertToList(struct getdns_list* list);
 
 PyObject*
 convertToDict(struct getdns_dict* dict) {
@@ -777,11 +740,9 @@ convertToDict(struct getdns_dict* dict) {
     }
 
     // try it as an IP
-    char* ipStr = getdns_dict_to_ip_string(dict);
-    if (ipStr) {
-        PyObject* res1 = Py_BuildValue("s", ipStr);
-        PyDict_SetItem(resultsdict1, PyString_FromString("IPSTRING"), res1);
-        return resultsdict1;
+    PyObject *ipdict = getdns_dict_to_ip_string(dict);
+    if (ipdict) {
+        return ipdict;
     }
 
     getdns_list* names;
@@ -887,8 +848,7 @@ convertToList(struct getdns_list* list) {
                 getdns_dict* dict = NULL;
                 getdns_list_get_dict(list, i, &dict);
                 PyObject *rl1 = convertToDict(dict);
-                PyObject *res1 = Py_BuildValue("O", rl1);
-                PyList_Append(resultslist1, res1);
+                PyList_Append(resultslist1, rl1);
                 break;
             }
             case t_list:
@@ -909,24 +869,48 @@ convertToList(struct getdns_list* list) {
 }
 
 // potential helper to get the ip string of a dict
-char* getdns_dict_to_ip_string(getdns_dict* dict) {
+
+PyObject * 
+getdns_dict_to_ip_string(getdns_dict* dict) {
+    getdns_bindata *addr;
+    getdns_bindata *type;
+    getdns_return_t r;
+    PyObject *addr_dict;
+    char *addr_string;
+    PyObject *pyaddr_string;
+
     if (!dict) {
         return NULL;
     }
-    getdns_bindata* data;
-    getdns_return_t r;
-    r = getdns_dict_get_bindata(dict, "address_type", &data);
+    r = getdns_dict_get_bindata(dict, "address_type", &type);
     if (r != GETDNS_RETURN_GOOD) {
         return NULL;
     }
-    if (data->size == 5 &&
-        (strcmp("IPv4", (char*) data->data) == 0 ||
-         strcmp("IPv6", (char*) data->data) == 0)) {
-        r = getdns_dict_get_bindata(dict, "address_data", &data);
+    if (type->size == 5 &&
+        (strcmp("IPv4", (char*) type->data) == 0 ||
+         strcmp("IPv6", (char*) type->data) == 0)) {
+        r = getdns_dict_get_bindata(dict, "address_data", &addr);
         if (r != GETDNS_RETURN_GOOD) {
+            PyErr_SetString(getdns_error, GETDNS_RETURN_GENERIC_ERROR_TEXT);
+            return NULL;
+        }
+        if ((addr_dict = PyDict_New()) == NULL)  {
+            PyErr_SetString(getdns_error, GETDNS_RETURN_GENERIC_ERROR_TEXT);
+            return NULL;
+        }
+        if (PyDict_SetItemString(addr_dict, "address_type", PyString_FromString((char *)type->data)) != 0)  {
+            PyErr_SetString(getdns_error, GETDNS_RETURN_GENERIC_ERROR_TEXT);
+            return NULL;
+        }
+        addr_string = getdns_display_ip_address(addr);
+        if ((pyaddr_string = PyString_FromString(addr_string)) == NULL)  {
+            PyErr_SetString(getdns_error, GETDNS_RETURN_GENERIC_ERROR_TEXT);
+            return NULL;
+        }
+        if (PyDict_SetItemString(addr_dict, "address_data", pyaddr_string) != 0)  {
+            PyErr_SetString(getdns_error, GETDNS_RETURN_GENERIC_ERROR_TEXT);
             return NULL;
         }
-        return getdns_display_ip_address(data);
     }
     return NULL;
 }
@@ -986,8 +970,7 @@ getFullResponse(struct getdns_dict *dict)
                 getdns_list* list = NULL;
                 getdns_dict_get_list(dict, (char*)nameBin->data, &list);
                 PyObject* rl1 = convertToList(list);
-                PyObject *res1 = Py_BuildValue("O", rl1);
-                PyDict_SetItem(resultslist, PyString_FromString((char*)nameBin->data), res1);
+                PyDict_SetItem(resultslist, PyString_FromString((char*)nameBin->data), rl1); 
                 break;
             }
             default:
